import com.tagtraum.beatunes.action.*
import java.awt.event.*
import java.awt.*
import javax.swing.*
import com.tagtraum.core.app.*
import org.slf4j.*;

// Hack to allow for "scanning" of a playlist
// Appears in "Controls" menu
// After started with "Scan" (Command/CTRL-Y) it must be stopped the same way with "Stop Scan",
// otherwise chaos ensues.
// It's recommended to turn off the coverpanel. See http://help.beatunes.com/discussions/questions/2792-how-to-unselect-a-song#comment_34465778  
class ScanAction extends BaseAction {
    
    private Timer timer
    private Component currentComponent
    private int currentRow
    private Action peekAction

    def String getId() {
        "Groovy.ScanAction"
    }

    def void init() {
        putValue(Action.NAME, "Scan")
        putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_Y, Toolkit.getDefaultToolkit().getMenuShortcutKeyMask()));
        peekAction = getApplication().getActionManager().getAction("audioplayer.peekselected")
    }

    def ActionLocation[] getActionLocations() {
        [new AbsoluteActionLocation(BeaTunesUIRegion.ITUNES_MENU,
            AbsoluteActionLocation.FIRST)]
    }

    def void actionPerformed(ActionEvent actionEvent) {
        if (getApplication().getPlayer().isPaused()) {
            // start scan
            putValue(Action.NAME, "Stop Scan")
            final KeyboardFocusManager manager = KeyboardFocusManager.getCurrentKeyboardFocusManager();
            currentComponent = manager.getPermanentFocusOwner();
            if (currentComponent instanceof JTable) {
                final JTable table = (JTable) currentComponent;
                currentRow = table.getSelectedRow();
            }
            peekAction.actionPerformed(actionEvent)
            // wait 10s in between songs
            timer = new Timer(10000, new ActionListener() {
                void actionPerformed(ActionEvent e) {
                    scanNext()
                }
            })
            timer.setRepeats(true)
            timer.start()
        } else {
            // stop scan
            stop()
        }
    }
    
    def void scanNext() {
        if (currentComponent instanceof JTable) {
            final JTable table = (JTable) currentComponent
            int rows = table.getRowCount()
            if (rows > currentRow+1) {
                currentRow++
                table.setRowSelectionInterval(currentRow, currentRow)
                table.requestFocusInWindow()
                SwingUtilities.invokeLater(new Runnable() {
                    void run() {
                        peekAction.actionPerformed(null)
                    }
                })
            } else {
                stop()
            }
        }
    }
    
    def void stop() {
        putValue(Action.NAME, "Scan")
        if (timer != null) timer.stop()
        getApplication().getPlayer().close()
    }
    
}
